@@ -1,18 +1,26 @@
-#####################################################################
+##############################################################################
 #
-# LDAPUserFolder	An LDAP-based user source for Zope
+# Copyright (c) 2000-2008 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
-# This software is governed by a license. See
-# LICENSE.txt for the terms of this license.
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
 #
-#####################################################################
-__version__='$Revision: 1410 $'[11:-2]
+##############################################################################
+""" The LDAPUserFolder class
+
+$Id: LDAPUserFolder.py 1485 2008-06-04 16:08:38Z jens $
+"""
 
 # General python imports
 import logging
 import os
 import random
 from sets import Set
+import sha
 import time
 import urllib
 
@@ -32,6 +40,7 @@ from OFS.SimpleItem import SimpleItem
 
 # LDAPUserFolder package imports
 from Products.LDAPUserFolder.LDAPDelegate import filter_format
+from Products.LDAPUserFolder.LDAPUser import NonexistingUser
 from Products.LDAPUserFolder.LDAPUser import LDAPUser
 from Products.LDAPUserFolder.SharedResource import getResource
 from Products.LDAPUserFolder.SimpleCache import SharedObject
@@ -133,11 +142,18 @@ class LDAPUserFolder(BasicUserFolder):
         auth_timeout = self.getCacheTimeout('authenticated')
         self._cache('authenticated').setTimeout(auth_timeout)
 
+        negative_timeout = self.getCacheTimeout('negative')
+        self._cache('negative').setTimeout(negative_timeout)
+
         # For older LDAPUserFolders which don't distinguish between uid
         # and login id attrs, provide a _uid_attr
         if not hasattr(self, '_uid_attr'):
             self._uid_attr = self._login_attr
 
+        # Delete old-style logger instances
+        if getattr(self, '_logger', None) is not None:
+            del self._logger
+
 
     def __init__(self, delegate_type='LDAP delegate'):
         """ Create a new LDAPUserFolder instance """
@@ -182,6 +198,7 @@ class LDAPUserFolder(BasicUserFolder):
         """ Clear all logs and caches for user-related information """
         self._cache('anonymous').clear()
         self._cache('authenticated').clear()
+        self._cache('negative').clear()
         self._misc_cache().clear()
 
     security.declarePrivate('_lookupuserbyattr')
@@ -490,6 +507,13 @@ class LDAPUserFolder(BasicUserFolder):
 
         return tuple(mv)
 
+    security.declareProtected(manage_users, 'getBinaryUserAttrs')
+    def getBinaryUserAttrs(self):
+        """ Return sequence of binary user attributes"""
+        schema = self.getSchemaDict()
+        bins = [x['ldap_name'] for x in schema if x.get('binary', '')]
+
+        return tuple(bins)
 
     security.declareProtected(manage_users, 'getUsers')
     def getUsers(self, authenticated=1):
@@ -644,9 +668,13 @@ class LDAPUserFolder(BasicUserFolder):
         """
         if not value:
             return None
-
+        
         cache_type = pwd and 'authenticated' or 'anonymous'
+        negative_cache_key = '%s:%s' % (value, sha.new(pwd or '').digest())
         if cache:
+            if self._cache('negative').get(negative_cache_key) is not None:
+                return None
+
             cached_user = self._cache(cache_type).get(value, pwd)
 
             if cached_user:
@@ -662,6 +690,7 @@ class LDAPUserFolder(BasicUserFolder):
 
         if user_dn is None:
             logger.debug('getUserByAttr: "%s=%s" not found' % (name, value))
+            self._cache('negative').set(negative_cache_key, NonexistingUser())
             return None
 
         if user_attrs is None:
@@ -669,6 +698,7 @@ class LDAPUserFolder(BasicUserFolder):
                 name, value
                 )
             logger.debug(msg)
+            self._cache('negative').set(negative_cache_key, NonexistingUser())
             return None
 
         if user_roles is None or user_roles == self._roles:
@@ -677,8 +707,8 @@ class LDAPUserFolder(BasicUserFolder):
                 )
             logger.debug(msg)
 
-        login_name = user_attrs.get(self._login_attr)
-        uid = user_attrs.get(self._uid_attr)
+        login_name = user_attrs.get(self._login_attr, '')
+        uid = user_attrs.get(self._uid_attr, '')
 
         if self._login_attr != 'dn' and len(login_name) > 0:
             if name == self._login_attr:
@@ -691,6 +721,7 @@ class LDAPUserFolder(BasicUserFolder):
                     user_dn, self._login_attr
                     )
             logger.debug(msg)
+            self._cache('negative').set(negative_cache_key, NonexistingUser())
             return None
 
         if self._uid_attr != 'dn' and len(uid) > 0:
@@ -700,6 +731,7 @@ class LDAPUserFolder(BasicUserFolder):
                     user_dn, self._uid_attr
                     )
             logger.debug(msg)
+            self._cache('negative').set(negative_cache_key, NonexistingUser())
             return None
 
         user_obj = LDAPUser( uid
@@ -1090,7 +1122,8 @@ class LDAPUserFolder(BasicUserFolder):
                 rec_dict = {}
 
                 for key, val in res_dicts[i].items():
-                    rec_dict[key] = val[0]
+                    if len(val) > 0:
+                        rec_dict[key] = val[0]
 
                 rec_dict['dn'] = dn
 
@@ -1331,6 +1364,7 @@ class LDAPUserFolder(BasicUserFolder):
                                 , friendly_name=''
                                 , multivalued=''
                                 , public_name=''
+                                , binary=''
                                 , REQUEST=None
                                 ):
         """ Add a schema item to my list of known schema items """
@@ -1340,6 +1374,7 @@ class LDAPUserFolder(BasicUserFolder):
                                 , 'friendly_name' : friendly_name
                                 , 'public_name' : public_name
                                 , 'multivalued' : multivalued
+                                , 'binary' : binary
                                 }
 
             self.setSchemaConfig(schema)
@@ -1453,7 +1488,9 @@ class LDAPUserFolder(BasicUserFolder):
             for attribute, names in self.getSchemaConfig().items():
                 attr_val = source.get(attribute, None)
 
-                if attr_val:
+                if names.get('binary', None) and attr_val:
+                    attr_dict['%s;binary' % attribute] = attr_val
+                elif attr_val:
                     attr_dict[attribute] = attr_val
                 elif names.get('public_name', None):
                     attr_val = source.get(names['public_name'], None)
@@ -1665,15 +1702,19 @@ class LDAPUserFolder(BasicUserFolder):
         """ Set a new attribute on the user record """
         schema = self.getSchemaConfig()
         prop_info = schema.get(prop_name, {})
+        is_binary = prop_info.get('binary', None)
 
         if isinstance(prop_value, str) or isinstance(prop_value, unicode):
-            if not prop_info.get('multivalued', ''):
+            if is_binary:
+                prop_value = [prop_value]
+            elif not prop_info.get('multivalued', ''):
                 prop_value = [prop_value.strip()]
             else:
                 prop_value = [x.strip() for x in prop_value.split(';')]
 
-        for i in range(len(prop_value)):
-            prop_value[i] = to_utf8(prop_value[i])
+        if not is_binary:
+            for i in range(len(prop_value)):
+                prop_value[i] = to_utf8(prop_value[i])
 
         cur_rec = self._delegate.search( base=user_dn
                                        , scope=self._delegate.BASE
@@ -1695,9 +1736,14 @@ class LDAPUserFolder(BasicUserFolder):
             else:
                 mod = self._delegate.DELETE
 
+            if is_binary:
+                attrs = { '%s;binary' % prop_name : prop_value }
+            else:
+                attrs = { prop_name : prop_value }
+
             err_msg = self._delegate.modify( dn=user_dn
                                            , mod_type=mod
-                                           , attrs={prop_name:prop_value}
+                                           , attrs=attrs
                                            )
 
             if not err_msg:
@@ -1723,7 +1769,10 @@ class LDAPUserFolder(BasicUserFolder):
             if source.has_key(attr):
                 new = source.get(attr, '')
                 if isinstance(new, str) or isinstance(new, unicode):
-                    if not attr_info.get('multivalued', ''):
+                    if attr_info.get('binary', ''):
+                        new = [new]
+                        attr = '%s;binary' % attr
+                    elif not attr_info.get('multivalued', ''):
                         new = [new.strip()]
                     else:
                         new = [x.strip() for x in new.split(';')]
@@ -1849,10 +1898,7 @@ class LDAPUserFolder(BasicUserFolder):
     security.declareProtected(manage_users, 'getCacheTimeout')
     def getCacheTimeout(self, cache_type='anonymous'):
         """ Retrieve the cache timout value (in seconds) """
-        if cache_type == 'authenticated':
-            return getattr(self, '_authenticated_timeout', 600)
-        else:
-            return getattr(self, '_anonymous_timeout', 600)
+        return getattr(self, '_%s_timeout' % cache_type, 600)
 
 
     security.declareProtected(manage_users, 'setCacheTimeout')
@@ -1867,10 +1913,7 @@ class LDAPUserFolder(BasicUserFolder):
         else:
             timeout = int(timeout)
 
-        if cache_type == 'authenticated':
-            self._authenticated_timeout = timeout
-        elif cache_type == 'anonymous':
-            self._anonymous_timeout = timeout
+        setattr(self, '_%s_timeout' % cache_type, timeout)
 
         self._cache(cache_type).setTimeout(timeout)
 
